/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAM_BENCHMARK_H_
#define EAM_BENCHMARK_H_

#include <time.h>

/* Measure the time of some operations, displaying a message on stderr. Note
   how EAM_START_BENCHMARK and EAM_STOP_BENCHMARK can be safely nested. This
   is especially useful for debugging. 
   The time is reported (and/or written into the given double variable) in
   seconds. */
#define EAM_START_BENCHMARK() \
  { const clock_t _time_before = clock()
#define EAM_STOP_BENCHMARK_AND_PRINT(text)          \
  fprintf(stderr, "%s: %g secs.\n",             \
          text,                                 \
  ((double) (clock() - _time_before)) / CLOCKS_PER_SEC); }
#define EAM_STOP_BENCHMARK_AND_ASSIGN(variable) \
  variable = \
    ((double) (clock() - _time_before)) / CLOCKS_PER_SEC; }
#define EAM_STOP_BENCHMARK_PRINT_AND_ASSIGN(text, variable)    \
  const clock_t _time_after = clock();                     \
  variable = ((double) (clock() - _time_before)) / CLOCKS_PER_SEC;      \
  fprintf(stderr, "%s: %g secs.\n", text, variable); } 

#endif // #ifndef EAM_BENCHMARK_H_
