/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2002 Luca Saiu
Minor updates by Luca Saiu, 2006

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with epsilon; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA. */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "malloc.h"
#include "command_line.h"

/* Commodity versions of malloc() and realloc() which abort instead of
   returning NULL when memory is exhausted: */
void* eam_xmalloc(size_t size){
  /* Try to allocate: */
  void* r = malloc(size);

  /* Check and return: */
  if(r == NULL)
    eam_fatal("eam_xmalloc(): memory exhausted");
  else
    return r;
}

void* eam_xrealloc(void* old_pointer, size_t new_size){
  /* Try to allocate: */
  void* r = realloc(old_pointer, new_size);

  /* Check and return: */
  if(r == NULL)
    eam_fatal("eam_xrealloc(): memory exhausted");
  else
    return r;
}

/* Trivial: just call eam_xmalloc() and clear: */
void* cmalloc(size_t size){
  void *r = eam_xmalloc(size);
  memset(r, 0, size);
  return r;
}
