/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2002, 2003 Luca Saiu
Some modifications by Luca Saiu in 2005 and 2006.

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with epsilon; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA. */

#ifndef EAM_STRINGMAP_H_
#define EAM_STRINGMAP_H_

#include <stdlib.h>		/* for size_t */
#include "types.h"

/* A map from strings to any objects: strings are compared according to their 
   values: */
struct eam_string_map;

/* A function used to destroy object pointed by the allocated datum. This is
   useful for destroying pointer structures used as hash data. Such a function
   should *not* destroy the datum itself, but only other pointed-to data, if any.
   The datum itself is always automatically destroyed. */
typedef void(*eam_string_map_datum_finalizer)(const void*);

/* A eam_string_map_datum_finalizer doing nothing. This is reasonable for 
   the common case of tables whose data don't contain pointers: */
void eam_default_string_map_datum_finalizer(const void*);

/* Keys can be large at will. */

/* Create a string map with the default size: */
struct eam_string_map *eam_create_string_map (size_t size_of_value,
                                          const eam_string_map_datum_finalizer datum_finalizer);
struct eam_string_map *eam_create_string_map_of_size (size_t size_of_value,
					      const int size,
					      const eam_string_map_datum_finalizer datum_finalizer);

void eam_destroy_string_map (struct eam_string_map *map);

/* This does *not* replace an already present value. If a value is already
   present then a warning is printed, as of now. */
void eam_insert_into_string_map (struct eam_string_map *map,
			     const char *key,
			     void *value);
void eam_multinsert_into_string_map (struct eam_string_map *map,
                                     const char *key,
                                     void *value);
void eam_remove_from_string_map (struct eam_string_map *map,
			     const char *key);
void *eam_access_string_map (struct eam_string_map *map,
			 const char *key);
int eam_is_string_map_defined_on (struct eam_string_map *map,
			      const char *key);

#endif /* #ifndef EAM_STRINGMAP_H_ */
