/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2002, 2005 Luca Saiu
Updated in 2006 by Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAM_TYPES_H_
#define EAM_TYPES_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <malloc.h>
#include "config.h"
//#include <dmalloc.h>

/* integer_t must be exactly wide as void*: */
#if SIZEOF_VOID_P == SIZEOF_SHORT
  #error void* is as wide as short... Is this a 16-bit machine?
#elif SIZEOF_VOID_P == SIZEOF_INT
  typedef int integer_t;
  typedef unsigned int unsigned_integer_t;
  #define INTEGER_T_FORMAT "%i"
  #define UNSIGNED_INTEGER_T_FORMAT "%u"
  #define HEXADECIMAL_INTEGER_T_FORMAT "%x"
  #define HEXADECIMAL_UNSIGNED_INTEGER_T_FORMAT "%x"
//#warning integer_t is an int
#elif SIZEOF_VOID_P == SIZEOF_LONG
  typedef long integer_t;
  typedef unsigned long unsigned_integer_t;
  #define INTEGER_T_FORMAT "%li"
  #define UNSIGNED_INTEGER_T_FORMAT "%lu"
  #define HEXADECIMAL_INTEGER_T_FORMAT "%lx"
  #define HEXADECIMAL_UNSIGNED_INTEGER_T_FORMAT "%lx"
//#warning integer_t is a long
#elif SIZEOF_VOID_P == SIZEOF_LONG_LONG
/* This can not happen if we don't really have long long: in this
   case long would have matched since the test is before this one */
  #warning This is strange, void* is as wide as long long...
  typedef long_long integer_t;
  typedef unsigned long_long unsigned_integer_t;
  #define INTEGER_T_FORMAT "%lli"
  #define UNSIGNED_INTEGER_T_FORMAT "%llu"
  #define HEXADECIMAL_INTEGER_T_FORMAT "%llx"
  #define HEXADECIMAL_UNSIGNED_INTEGER_T_FORMAT "%llx"
//#warning integer_t is a long long
#else
  /* This should not happen */
  #error There is no integer type as wide as void*
#endif

/* A void*, an integer_t or a float_t must fit exactly in a word_t: */
typedef void* word_t;

/* Essentially only for aesthetics: */
typedef void* pointer_t;

/* The C syntax for pointer-to-function types is unforgivably ugly, and
   a typedef doesn't allow to define a recursive type in any natural way.
   We have to resort to ugly syntax, and casts. Oh, well. */

/* We implement a continuation as a (pointer to a) function taking no
   parameters and returning a pointer which may be casted to another 
   continuation: */
typedef word_t (*eam_continuation_t)(void);

#endif // #ifndef EAM_TYPES_H_
