/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <assert.h>
#include <sys/wait.h>

#include "../../common/types.h"

/* We reserve the register REGISTER for the global x: */
register word_t x asm ( REGISTER );
volatile integer_t copy_of_x;
/*
register word_t x1 asm ( "%r13" );
register word_t x2 asm ( "%r14" );
register word_t x3 asm ( "%r15" );
register word_t x4 asm ( "%r16" );
register word_t x5 asm ( "%r17" );
register word_t x6 asm ( "%r18" );
register word_t x7 asm ( "%r19" );
register word_t x8 asm ( "%r20" );
register word_t x9 asm ( "%r21" );
register word_t x10 asm ( "%r22" );
register word_t x11 asm ( "%r23" );
register word_t x12 asm ( "%r24" );
register word_t x asm ( "%r25" );
register word_t x14 asm ( "%r26" );
register word_t x15 asm ( "%r27" );
register word_t x16 asm ( "%r28" );
register word_t x17 asm ( "%r29" );
register word_t x18 asm ( "%r30" );
*/


word_t do_a_lot_of_stuff_in_another_compilation_unit(const word_t x, 
                                                     const integer_t increment,
                                                     ...)
/* GCC isn't currently able to inline this, but we want to be safe now and
   in the future. */
  __attribute__(( noinline )); 

/* When trying with "wrong" registers I noticed strange things happening with printf().
   Let's test it: */
integer_t printf_test(integer_t n, void *f){
  volatile word_t another_copy_of_x = (word_t)42;
  char s[1000];
  sprintf(s, "%p", (word_t)x);
  sscanf(s, "%p", (word_t*)&another_copy_of_x);
  assert(x == another_copy_of_x);
  assert((integer_t)x == copy_of_x);
  printf("    This one: "INTEGER_T_FORMAT" should be equal to...\n", (integer_t)x);
  printf(" ...this one: "INTEGER_T_FORMAT"\n", (integer_t)another_copy_of_x);
  printf("and this one: "INTEGER_T_FORMAT"\n\n", (integer_t)copy_of_x);
  if(n != 0){
    const integer_t increment = (rand() % 2000) - 1000;
    x = do_a_lot_of_stuff_in_another_compilation_unit(x, increment, 245, -345, 1, 0);
    copy_of_x += increment;
    // the recursive call must not be sibling-call optimized:
    integer_t(*g)(integer_t, void*) = f;
    g(n - 1, f);
    printf("Ok "INTEGER_T_FORMAT"\n", (integer_t)x); 
    return (integer_t)x;
  }
  return (integer_t)x;
}

int main(){
  srand(time(NULL));
  unsigned_integer_t r = (unsigned_integer_t)(rand() % RAND_MAX);
  x = (word_t)r;
  copy_of_x = (integer_t)x;
  
  assert(x == (word_t)r);
  assert((unsigned_integer_t)x == r);
  
  x = do_a_lot_of_stuff_in_another_compilation_unit(x, -42,
                                                    10, 20, 324, 324, 0);
  x = do_a_lot_of_stuff_in_another_compilation_unit(x, +42,
                                                    -234, 245, -345, 1, 0);
  
  unsigned_integer_t increment = rand() % 50;
  x = (word_t)(((unsigned_integer_t)x) + increment);
  
  assert(x == (word_t)(r + increment));
  assert((((unsigned_integer_t)x) - increment) == r);
  x = do_a_lot_of_stuff_in_another_compilation_unit(x, 0, -1, -2, -3, 0);
  assert(x == (word_t)(r + increment));
  assert((((unsigned_integer_t)x) - increment) == r);

  
  x = (word_t)(((unsigned_integer_t)x) - increment);

  assert((integer_t)x == copy_of_x);
  
  /* Is this register saved on a context switch? This assumes an 'ls' takes
     less than 5 seconds. Let's execute it once first, to make the next call
     faster: */
  system("ls");
  pid_t pid = fork();
  assert(pid != -1);
  if(pid == 0){
    sleep(5);
    system("ls");
    exit(0);
  }
  else
    assert(waitpid(pid, NULL, 0u) != -1);
  assert((integer_t)x == copy_of_x);

  printf_test(100, printf_test);
/*   integer_t i; */
/*   volatile word_t another_copy_of_x = (word_t)42; */
/*   for(i = 0; i < 1000; i++){ */
/*     char s[1000]; */
/*     sprintf(s, "%p", (word_t)x); */
/* x = do_a_lot_of_stuff_in_another_compilation_unit(x, 0, -1, -2, -3, 0); */
/*     sscanf(s, "%p", (word_t*)&another_copy_of_x); */
/*     assert(x == another_copy_of_x); */
/*     assert((integer_t)x == copy_of_x); */
/*     printf("   This one: "INTEGER_T_FORMAT" should be equal to...\n", (integer_t)x); */
/*     printf("...this one: "INTEGER_T_FORMAT"\n", (integer_t)another_copy_of_x); */
/*     x++; */
/*     copy_of_x++; */
/*   } */
  
  return 0;
}
