/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAMGC_COMPILE_TIME_PARAMETERS_H_
#define EAMGC_COMPILE_TIME_PARAMETERS_H_


/* The user should not directly #include this file. */

#include "../../common/types.h"


/* The number of bits used for the offset within a homogeneous page in a
   pointer: */
#define EAMGC_PAGE_OFFSET_WIDTH 19//18//19//8//10//11//16//11 //16//20 //20//20//20//19//8 //20 //8 //10 //17 //14//17//17//18 /* in bits */

/* These are sizes of hash tables; it's better if these are prime numbers,
   since this statistically minimizes collisions: */
#define EAMGC_SIZE_OF_SET_OF_HOMOGENEOUS_PAGES 10007 
#define SIZE_OF_SET_OF_LARGE_OBJECTS     19997

/* The initial minimum and maximum heap size limits (note that the minimum heap
   size for the large heap is always zero). See the comments in statistics.h
   for more information. Sizes are in words: */
#define EAMGC_INITIAL_MINIMUM_HOMOGENEOUS_HEAP_SIZE \
  ((unsigned_integer_t)(1u << 24u) / (unsigned_integer_t)sizeof(word_t)) // 16Mb
#define EAMGC_INITIAL_MAXIMUM_HOMOGENEOUS_HEAP_SIZE \
  ((unsigned_integer_t)(1u << 26u) / (unsigned_integer_t)sizeof(word_t)) // 64Mb
#define EAMGC_INITIAL_MAXIMUM_LARGE_HEAP_SIZE \
  ((unsigned_integer_t)(1u << 24u) / (unsigned_integer_t)sizeof(word_t)) // 16Mb
/* The minimum allowed upper limit: */
#define EAMGC_MINIMUM_MAXIMUM_LARGE_HEAP_SIZE \
  ((unsigned_integer_t)(1u << 18u) / (unsigned_integer_t)sizeof(word_t)) // 256Kb

/* The GC refuses to make the heap grow further when the system declares thare
   are less than EAMGC_GROWTH_SAFETY_MARGIN free words in system memory. See
   the comment in statistics.c: */
#define EAMGC_GROWTH_SAFETY_MARGIN \
  ((unsigned_integer_t)(1u << 24u) / (unsigned_integer_t)sizeof(word_t)) // 16Mb

/* A closely related safety margin. This is the minimum space, in words, which
   we don't allocate for our heap and we leave to the the other processes, when
   we're expanding the heap. */
#define EAMGC_MINIMUM_HEAP_SPACE_FOR_OTHER_PROCESSES \
  ((unsigned_integer_t)(1u << 26u) / (unsigned_integer_t)sizeof(word_t)) // 64Mb

/* The optimal ratio between live heap size and total heap size. The GC heuristics
   try to adjust the heap size to respect this ratio, except for very large or
   very small heaps (keeping the ratio constant in those cases would make
   performance *worse*, causing trashing or unnecessary deallocation followed by
   allocation). The ratio must be in (0, 1):  */
#define EAMGC_LIVE_TO_TOTAL_RATIO 0.01//0.00001//0.01

/* Generic pools are created for objects of *all* the sizes in the closed range
   [1 .. EAMGC_MAXIMUM_EXACTLY_SUPPORTED_GENERIC_OBJECT_SIZE]: please e-mail
   me if you find a better name: :-).
   This doesn't need to be a power of two. */
#define EAMGC_MAXIMUM_EXACTLY_SUPPORTED_GENERIC_OBJECT_SIZE 32//4//32

/* How many words is the largest generic object wide. This must be a power of
   two: */
#define EAMGC_MAXIMUM_GENERIC_OBJECT_SIZE 128//16//128


/* These are automatically computed from other constants: */
#define EAMGC_PAGE_SIZE_IN_CHARS (1 << EAMGC_PAGE_OFFSET_WIDTH)
#define EAMGC_PAGE_SIZE (EAMGC_PAGE_SIZE_IN_CHARS / sizeof(word_t)) /* in words */

#endif // #ifndef EAMGC_COMPILE_TIME_PARAMETERS_H_
