/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "malloc_emulation.h"
#include "../../common/c_context.h"

/* The #define changing main()'s name is just for the user: here we need
   the compiler to emit a main() function which is actually called "main".
   It will be the *real* program entry point: */
#undef main

/* The *real* program entry point: */
int main(int argc, char **argv){
  /* Initialize the collector and make generic pools: */
  eamgc_initialize_garbage_collector();
  eamgc_create_generic_pools();
  
  /* Use C stack and registers as roots, for conservative pointer finding: */
  eamgc_register_root_tracing_function(eamgc_trace_c_context_roots);
  
  /* At exit time there is no need to destroy everything. Just finalize what
     requires finalization: */
  atexit(eamgc_finalize_large_objects);
  
  /* This funny continuation-passing style interface is needed so that a
     stack bottom approximation can be safely saved. See the comments in
     c_context.h. Ok, now we just have to save the stack bottom and 
     call the "main" function supplied by the user: */
  int returned_value =
    eam_save_stack_bottom_and_call_main(__eamgc_what_the_user_thinks_its_main, 
                                        argc, argv);
  
  /* Return the exit value we kept: */
  return returned_value;
}
