/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "gc.h"

void __called_from_main(void);
void __what_the_user_thinks_its_main(void);

int main(int argc, char **argv){
  eamgc_initialize_garbage_collector_and_continue(__called_from_main);
  return 0;
}

void __called_from_main(void){
  eamgc_register_root_tracing_function(eamgc_trace_c_context_roots);
  //  eamgc_create_generic_pools();
  //  alloca(10000);
  __what_the_user_thinks_its_main();
}

#define malloc eamgc_malloc
#define calloc eamgc_calloc
#define realloc eamgc_realloc
#define free eamgc_free
#define main __what_the_user_thinks_its_main


struct eam_tree{
  struct eam_tree *left;
  integer_t value;
  struct eam_tree *right;
};

static struct eamgc_homogeneous_pool *trees_pool;
static struct eamgc_large_pool *trees_large_pool;

void trace_tree(const word_t tree_as_void_star){
  const struct eam_tree *tree = (const struct eam_tree *)tree_as_void_star;
  eamgc_trace_object((word_t)tree->left);
  eamgc_trace_object((word_t)tree->right);
}

void trace_tree_as_large_object(const word_t tree_as_void_star,
                                const unsigned_integer_t size){
  /* The size is contant, and unused. */
  trace_tree(tree_as_void_star);
}

void finalize_tree_as_large_object(const word_t tree_as_void_star,
                                   const unsigned_integer_t size){
  const struct eam_tree *tree = (const struct eam_tree *)tree_as_void_star;
  fprintf(stderr, "A node with value "UNSIGNED_INTEGER_T_FORMAT" is dying\n",
          (unsigned_integer_t)tree->value);
}

struct eam_tree* make_tree(struct eam_tree * left,
                       int value,
                       struct eam_tree * right){
  struct eam_tree* r = (struct eam_tree*)
    //malloc(sizeof(struct eam_tree));
    eamgc_allocate_from_homogeneous_pool(trees_pool);
    //eamgc_allocate_from_large_pool(trees_large_pool, 3u);
  r->left = (struct eam_tree*)left;
  r->value = value;
  r->right = (struct eam_tree*)right;
  return r;
}

struct eam_tree* insert_functional(const int x, struct eam_tree *tree){
  if(tree == NULL)
    return make_tree(NULL,
                     x,
                     NULL);
  else if(x <= tree->value)
    return make_tree(insert_functional(x, tree->left), 
                     tree->value,
                     tree->right);
  else // x > tree->value
    return make_tree(tree->left,
                     tree->value,
                     insert_functional(x, tree->right));
}

struct eam_tree* insert_destructive(const int x, struct eam_tree *tree){
  if(tree == NULL)
    return make_tree(NULL,
                     x,
                     NULL);
  else if(x <= tree->value){
    tree->left = insert_destructive(x, tree->left);
    return tree;
  }
  else{ // x > tree->value
    tree->right = insert_destructive(x, tree->right);
    return tree;
  }
}

void dump_tree(const struct eam_tree* tree){
  if(tree == NULL)
    return;
  else{
    dump_tree(tree->left);
    printf(INTEGER_T_FORMAT" ", (integer_t)tree->value);
    dump_tree(tree->right);
  }
}

#define N 10000000

#define MIN 1
#define MAX (N * 10)
#define SOMETHING (MIN + (rand() % (MAX - MIN)))

void main(){
  trees_pool =
    eamgc_create_homogeneous_pool(3u,
                               1u,
                               NULL,
                               trace_tree
                               //eamgc_homogeneous_trace_everything_function
                               );

  trees_large_pool =
    eamgc_create_large_pool(1u,
                         NULL,
                         trace_tree_as_large_object, 
                         //eamgc_large_trace_everything_function,
                         //finalize_tree_as_large_object
                         NULL
                         );
  int i;
  struct eam_tree *tree = NULL;
  srand(1);

  for(i = 0; i < N; i++)
    tree = insert_functional(SOMETHING, tree);

  //for(i = 0; i < N * 100; i++)
  //  malloc(rand() % 4 + 1 * (rand() % 16 + 1));
    //tree = insert_functional(SOMETHING, tree);
  //dump_tree(tree); printf("\n");
}
