/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include <assert.h>

#define SIZE 200000
#define LIST_LENGTH (1 + SIZE)
//#define N (2000001 / LIST_LENGTH) // don't change this
#define N (200000001 / LIST_LENGTH) // don't change this

#if defined(BOEHM)
  #include "/usr/include/gc.h"
  #define malloc GC_MALLOC
  #define realloc GC_REALLOC
  #define free GC_FREE
#elif defined(EAMGC)
  #include "malloc_emulation.h"
#else
  // #include nothing more. We'll just use malloc() and free()
#endif

struct my_list{
  int car;
  struct my_list *cdr;
}; // struct

struct my_list* my_eam_cons(const int x, struct my_list *xs){
  struct my_list *r = (struct my_list*)malloc(sizeof(struct my_list));
  r->car = x;
  r->cdr = xs;
  return r;
}

struct my_list* make_list(void){
  struct my_list *r = NULL;
  int i;
  for(i = LIST_LENGTH; i >= 0; i--)
    r = my_eam_cons(i, r);
  return r;
}

void dump_list(const struct my_list *list){
  const struct my_list *p;
  for(p = list; p != NULL; p = p->cdr)
    printf("%i\n", p->car);
}

void check_list(const struct my_list *list){
  const struct my_list *p;
  int i;
  for(p = list, i = 0; p != NULL; p = p->cdr, i++)
    assert(i == p->car);
  fprintf(stderr, ".");
}

struct my_list* my_eam_reverse(const struct my_list *list){
  struct my_list *r = NULL;
  const struct my_list *p;
  for(p = list; p != NULL; p = p->cdr)
    r = my_eam_cons(p->car, r);
  return r;
}

void destroy_list(struct my_list *list){
  struct my_list *p = list;
  while(p != NULL){
    struct my_list *next = p->cdr;
    free(p);
    p = next;
  } // while
}

int main(int argc, char **argv){
#ifdef BOEHM
  printf("Testing with the conservative pointer-finding GC by Boehm-Demers-Weiser...\n");
  //GC_enable_incremental();
#elif defined(EAMGC)
  printf("Testing with the epsilon Abstract Machine GC by Luca Saiu...\n");
  eamgc_dump_heap_size();
#else
  printf("Testing with malloc() and free() from the libc\n");
#endif

  printf("Starting the test...\n");
  int i;
  for(i = 0; i < N; i++){
    struct my_list *xs = make_list();
    check_list(xs);

    //dump_list(xs);
#if (! defined(BOEHM)) && (! defined(EAMGC))
    destroy_list(xs);
#endif
  } // while

  printf("  done.\n");

  return 0;
}
