/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "stack.h"
#include "fast_stack.h"
#include <assert.h>

static word_t st_fast_stack;
static word_t st_fast_stack_top_pointer;
static word_t st_fast_stack_allocated_size;

static word_t sf_fast_stack;
static word_t sf_fast_stack_frame_pointer;
static word_t sf_fast_stack_allocated_size;

#define N 10000

void dump(void){
  integer_t j;
  fprintf(stderr, "[ ");
  for(j = 0; j < N; j++)
    fprintf(stderr, ""INTEGER_T_FORMAT" ", *(((integer_t*)st_fast_stack) + j));
  fprintf(stderr, "]\n");
}

integer_t test1(void){
  integer_t i;
  for(i = 0; i < N; i++)
    eam_push_word_onto_fast_stack((word_t)i, st);
  integer_t sum = 0;
  for(i = 0; i < N; i++)
    sum += (integer_t)eam_pop_word_from_fast_stack(st);
  return sum;
}

integer_t test2(void){
  integer_t i;
  for(i = 0; i < N; i++)
    eam_store_word_into_fast_stack(i, (word_t)i, sf);
  integer_t sum = 0;
  for(i = 0; i < N; i++)
    sum += (integer_t)eam_load_word_from_fast_stack(i, sf);
  return sum;
}

#define M 10000

int main(){
  fprintf(stderr, "Testing fast stacks...\n");
  eam_initialize_stacks();
  
  eam_create_fast_stack_with_frame_pointer(10240, sf);
  eam_create_fast_stack_with_top_pointer(10240, st);
  
  eam_resize_fast_stack_with_frame_pointer(16384, sf);
  eam_resize_fast_stack_with_top_pointer(16384, st);

  eam_push_word_onto_fast_stack((word_t)12345, st);
  assert(eam_pop_word_from_fast_stack(st) == (word_t)12345);

  integer_t i;
  integer_t sum_of_sums = 0;
  for(i = 0; i < M; i++)
    sum_of_sums += test1();
  fprintf(stderr, INTEGER_T_FORMAT"\n", sum_of_sums);
  
  sum_of_sums = 0;
  for(i = 0; i < M; i++)
    sum_of_sums += test2();
  fprintf(stderr, INTEGER_T_FORMAT"\n", sum_of_sums);
  
  eam_destroy_fast_stack(sf);
  eam_destroy_fast_stack(st);

  eam_finalize_stacks(); 
  fprintf(stderr, "Done.\n");
  return 0;
}
