/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "stack.h"
#include "memory_stack.h"
#include <assert.h>

static struct eam_memory_stack *s;

#define N 10000

integer_t test1(void){
  integer_t i;
  for(i = 0; i < N; i++)
    eam_push_word_onto_memory_stack((word_t)i, s);
  integer_t sum = 0;
  for(i = 0; i < N; i++)
    sum += (integer_t)eam_pop_word_from_memory_stack(s);
  return sum;
}

integer_t test2(void){
  integer_t i;
  for(i = 0; i < N; i++)
    eam_store_word_into_memory_stack(i, (word_t)i, s);
  integer_t sum = 0;
  for(i = 0; i < N; i++)
    sum += (integer_t)eam_load_word_from_memory_stack(i, s);
  return sum;
}

#define M 10000

int main(){
  fprintf(stderr, "Testing memory stacks...\n");
  eam_initialize_stacks();
  
  s = eam_create_memory_stack(10240);
  
  eam_resize_memory_stack(16384, s);

  eam_push_word_onto_memory_stack((word_t)12345, s);
  assert(eam_pop_word_from_memory_stack(s) == (word_t)12345);

  integer_t i;
  integer_t sum_of_sums = 0;
  for(i = 0; i < M; i++)
    sum_of_sums += test1();  
  fprintf(stderr, INTEGER_T_FORMAT"\n", sum_of_sums);

  sum_of_sums = 0;
  for(i = 0; i < M; i++)
    sum_of_sums += test2();  
  fprintf(stderr, INTEGER_T_FORMAT"\n", sum_of_sums);

  eam_destroy_memory_stack(s);

  eam_finalize_stacks();
  fprintf(stderr, "Done.\n");
  return 0;
}
