/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>
#include "../common/types.h"
#include "../common/environment.h"
#include "eaml.h"
#include "symbol_table.h"
#include "eamlparser.h"
#include "generate_c.h"
#include "import_graph.h"
#include "../common/filesystem.h"
#include "../common/command_line.h"
#include "../common/malloc.h"
#include "eamc.h"

static void manage_command_line(int *argc, char **argv){
  /* Setup the "generic" part of command line support: */
  eam_set_program_name("eamc (" PACKAGE_NAME ")");
  eam_set_general_help_message("Compile and link eAML into native code, calling GCC.");
  eam_set_synopsis_string("eamc [OPTIONS] FILE1 ... FILEn");
  eam_set_version_string(VERSION_STRING);
  eam_set_copyright_string(COPYRIGHT_STRING);
  eam_set_license_message(LICENSE_STRING);
  eam_set_bug_reporting_message(BUG_REPORTING_MESSAGE);
  
  /* Add specific options: */
  static const char *default_search_path = ".";
  eam_add_option_with_nonoptional_parameter(
    "eaml-path",
    'p',
    eam_getenv_with_default("EAM_COMPILATION_UNIT_PATH",
                            default_search_path),
    "compilation unit search path");
  /*
  eam_add_option_with_nonoptional_parameter(
    "repository-path",
    'r',
    eam_getenv_with_empty_default("EAM_REPOSITORY_PATH"),
    "repository search path");
  */
  eam_add_toggle_option(
    "link",
    '\0',
    1,
    "link into an executable");
  
  /* Manage the actual command line: */
  eam_parse_command_line(argc, argv);
  
  //eam_show_options_status();
}

int main(int argc, char **argv){
  /* Support command line options: */
  manage_command_line(&argc, argv);
  
  /* Set the search path for .eaml files: */
  char *eaml_search_path_as_string =
    eam_long_option_to_value("eaml-path");
  struct eam_list *eaml_search_path =
    eam_parse_search_path(eaml_search_path_as_string);
  
  /* Set the search path for repositories: */
  /*
  char *repository_search_path_as_string =
    eam_long_option_to_value("repository-path");
  struct eam_list *repository_search_path = 
    eam_parse_search_path(repository_search_path_as_string);
  */
  
  /* Resolve all the directly referred .eaml files (i.e. not the ones
     obtained by resolving dependencies): */
  struct eam_list *eaml_files = 
    resolve_eaml_parameters(argc, argv, eaml_search_path);
  
  /* Parse the given eAML files, look for the files referred by them, parse
     also them, and so on until a fix point is reached. Return the list of
     all obtained compilation units: */
  struct eam_list *compilation_units =
    eaml_files_to_compilation_units(eam_files);
  
  
  /* Resolve all the directly referred Libtool library names: */
  //struct eam_list *library_files =
  //  resolve_library_parameters(argc, argv, repository_search_path);
  
  /* Ok, if everything went smooth till here we can start translating.
     First make the temporary directory: */
  char *temporary_directory = eam_create_temporary_directory_within(".", 0);//1);
  fprintf(stderr, "The temporary directory is %s\n", temporary_directory);
  
  fprintf(stderr, ".eaml files:   ");
  struct eam_list *p;
  for(p = eaml_files; ! eam_null(p); p = eam_cdr(p))
    fprintf(stderr, "%s ", (char*)eam_car(p));
  fprintf(stderr, "\n");
  
  
  /*  
  fprintf(stderr, "Library files: ");
  for(p = library_files; ! eam_null(p); p = eam_cdr(p))
    fprintf(stderr, "%s ", (char*)eam_car(p));
  fprintf(stderr, "\n");
  */
  return 0;
}
