/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "eaml.h"
#include "symbol_table.h"
#include "eamlparser.h"
#include "generate_c.h"
#include "../common/filesystem.h"

int main(int argc, char **argv){
  if(argc != 2)
    FATAL("there must be exactly one argument");
  
  /* Parse the given compilation unit and dump its exported symbols: */
  struct eam_compilation_unit *compilation_unit = eam_parse_compilation_unit(argv[1]);
  struct eam_symbol_table *symbol_table = eam_create_symbol_table();
  eam_fill_symbol_table_from_compilation_unit(symbol_table, compilation_unit, 1);
  if(eam_dump_exported_symbols_of_symbol_table(
        symbol_table,
	eam_replace_extension(compilation_unit->mangled_name,
			  SYMBOL_TABLE_EXTENSION)) != 0)
    FATAL("could not dump symbol table");
  
  return EXIT_SUCCESS;
}
