/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2003, 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "../common/types.h"
#include "../common/common_text.h"
#include "../common/command_line.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <limits.h>

#define SHOW_SIZE_OF_TYPE(type) \
  printf("  Objects of type " #type " are "UNSIGNED_INTEGER_T_FORMAT" bits wide.\n", (unsigned_integer_t)(sizeof(type) * CHAR_BIT))

static void show_types(){
  printf("  A C char is "UNSIGNED_INTEGER_T_FORMAT" bits wide.\n",
         (unsigned_integer_t)(CHAR_BIT));
  SHOW_SIZE_OF_TYPE(word_t);
  SHOW_SIZE_OF_TYPE(pointer_t);
  SHOW_SIZE_OF_TYPE(integer_t);
  //SHOW_SIZE_OF_TYPE(wide_integer_t);
  //SHOW_SIZE_OF_TYPE(float_t);
  //SHOW_SIZE_OF_TYPE(wide_float_t);
  //SHOW_SIZE_OF_TYPE(wide_wide_float_t);
}

static void show_paths(){
  printf("  eAM libraries are installed in %s\n",
	 "$EPSILON_LIBRARIES_PATH To do: fix this");
  printf("  eAM interfaces are installed in %s\n",
	 "$EPSILON_INTERFACES_PATH To do: fix this");
  printf("  eAM C libraries are installed in %s\n",
	 "$EPSILON_C_LIBRARIES_PATH To do: fix this");
}


static void show_line(){
  /* printf("---------------------------------\n"); */
}

static void show_everything(){
  show_line();

  printf("Information about eAM types:\n");
  show_types();
  show_line();

  printf("Information about paths:\n");
  show_paths();
  show_line();
}

int main(int argc, char** argv){
  /* Define command line behavior: */
  eam_set_program_name ("eam-config (" PACKAGE_NAME ")");
  eam_set_general_help_message("Show the configuration of the epsilon Abstract Machine");
  eam_set_synopsis_string ("eam-config [OPTIONS]");
  eam_set_version_string (VERSION_STRING);
  eam_set_copyright_string (COPYRIGHT_STRING);
  eam_set_license_message (LICENSE_STRING);
  eam_set_bug_reporting_message (BUG_REPORTING_MESSAGE);

  eam_add_toggle_option ("types", 't', 0,
                     "Show only types information");
  eam_add_toggle_option ("paths", 'p', 0,
                     "Show only paths information");
  
  /* Parse command-line and check its correctness: */
  eam_parse_command_line(&argc, argv);

  if(argc != 1)
    eam_command_line_fatal("There must be no command-line arguments");
  
  if(eam_long_option_to_value("types"))
    show_types();
  else if(eam_long_option_to_value("paths"))
    show_paths();
  else /* default: no command-line options */
    show_everything();
  
  return EXIT_SUCCESS;
} /* main */
