/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


/* This header allows to use the collector with fully conservative 
   pointer-finding as a mostly transparent and compatible replacement
   for the standard libc dynamic allocation features.
   It redefines malloc() and friends as macros, replacing them
   with calls to the matching GC functions for generic pools. It's
   important that this header is #include'd from *every* compilation unit
   using dynamic allocation.
   It's allowed to also #include the header file declaring or #defin'ing
   malloc(), either before of after this header; this header is written to
   "win" against the libc headers definitions.
   
   The user must write a main() function in the two parameters version,
   (and of course returning int, as required by the C99 Standard).
   
   There's no need for the usual xmalloc() hack, because the GC allocation
   primitives *never* return NULL.
   
   It's important that this header is *not* #include'd by the main GC
   header, so that users only enable it when explicitly needed. */

#ifndef EAMGC_MALLOC_EMULATION_H_
#define EAMGC_MALLOC_EMULATION_H_

/* #include all the standard libc headers which may #define malloc() and
   friends. The purpose of this is exploiting the usual preprocessor trick
   to avoid multiple inclusion, so that #includ'ing the standard headers
   *after* this one doesn't hide our #defin'itions: */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>

/* Of course we need the GC interface: */
#include "gc.h"
#include "generic_pools.h" // for eamgc_malloc() and friends

/* #undef'ine malloc() and friends, just in case they were #define'd as
   macros rather then declared as functions in standard headers. */
#ifdef malloc
#undef malloc
#endif // #ifdef malloc
#ifdef calloc
#undef calloc
#endif // #ifdef calloc
#ifdef realloc
#undef realloc
#endif // #ifdef realloc
#ifdef free
#undef free
#endif // #ifdef free

/* Ok, now we can safely re-#define names without even a compiler warning: */
#define malloc eamgc_malloc
#define calloc eamgc_calloc
#define realloc eamgc_realloc
#define free eamgc_free

/* Hide GC initialization and finalization to the user, who just writes a
   normal main(): */
#define main __eamgc_what_the_user_thinks_its_main

/* This prototype forces the user to respect the interface we need; otherwise
   compilation fails: */
int __eamgc_what_the_user_thinks_its_main(int argc, char **argv);

#endif // #ifndef EAMGC_MALLOC_EMULATION_H_
